@@ -2,11 +2,11 @@ package Mojolicious::Plugin::AssetPack;
 
 =head1 NAME
 
-Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass and javascript files
+Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascript and coffeescript files
 
 =head1 VERSION
 
-0.10
+0.11
 
 =head1 SYNOPSIS
 
@@ -17,7 +17,7 @@ In your application:
   plugin 'AssetPack';
 
   # define assets: $moniker => @real_assets
-  app->asset('app.js' => '/js/foo.js', '/js/bar.js');
+  app->asset('app.js' => '/js/foo.js', '/js/bar.js', '/js/baz.coffee');
   app->asset('app.css' => '/css/foo.less', '/css/bar.scss', '/css/main.css');
 
   # you can combine with assets from web
@@ -43,13 +43,21 @@ Or if you need to add the tags manually:
 
 See also L</register>.
 
+=head1 ENVIRONMENT
+
+=head2 MOJO_ASSETPACK_NO_CACHE
+
+If true, convert the assets each time they're expanded, instead of once at
+application start (useful for development). Has no effect when L</minify> is
+enabled.
+
 =head1 DESCRIPTION
 
 =head2 Production mode
 
-This plugin will compress scss, less, css and javascript with the help of
-external applications on startup. The result will be one file with all the
-sources combined. This file is stored in L</Packed directory>.
+This plugin will compress scss, less, css, javascript and coffeescript with the
+help of external applications on startup. The result will be one file with all
+the sources combined. This file is stored in L</Packed directory>.
 
 The files in the packed directory will have a checksum added to the
 filename which will ensure broken browsers request a new version once the
@@ -70,9 +78,14 @@ TIP! Make morbo watch your less/sass files as well:
 
   $ morbo -w lib -w templates -w public/sass
 
+You can also set the L</MOJO_ASSETPACK_NO_CACHE> environment variable to 1 to
+convert your less/sass/coffee files each time their asset directive is expanded
+(only works when L</minify> is disabled).
+
 =head2 Preprocessors
 
-This library tries to find default preprocessors for less, scss, js and css.
+This library tries to find default preprocessors for less, scss, js, coffee
+and css.
 
 NOTE! The preprocessors require optional dependencies to function properly.
 Check out L<Mojolicious::Plugin::AssetPack::Preprocessors/detect> for more
@@ -94,12 +107,13 @@ use Mojolicious::Plugin::AssetPack::Preprocessors;
 use File::Basename qw( basename );
 use File::Spec::Functions qw( catfile );
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 our %MISSING_ERROR = (
   default => '%s has no preprocessor. https://metacpan.org/pod/Mojolicious::Plugin::AssetPack::Preprocessors#detect',
   less => '%s require "less". http://lesscss.org/#usage',
   sass => '%s require "sass". http://sass-lang.com/install',
   scss => '%s require "sass". http://sass-lang.com/install',
+  coffee => '%s require "coffee". http://coffeescript.org/#installation',
 );
 
 =head1 ATTRIBUTES
@@ -123,6 +137,7 @@ unless a L<static directory|Mojolicious::Static/paths> is writeable.
 has minify => 0;
 has preprocessors => sub { Mojolicious::Plugin::AssetPack::Preprocessors->new };
 has out_dir => sub { File::Spec::Functions::catdir(File::Spec::Functions::tmpdir(), 'mojo-assetpack') };
+has _no_cache => 0;
 
 =head2 rebuild
 
@@ -158,28 +173,48 @@ sub add {
   if($self->minify) {
     $self->process($moniker => @files);
   }
+  elsif ($self->_no_cache) {
+    # Do nothing, as the assets will be processed each time in expand.
+  }
   else {
-    for my $file (@files) {
-      next unless $file =~ /\.(less|s[ac]ss)$/;
-      my $moniker = basename $file;
-      $moniker =~ s/\.\w+$/.css/;
-      $self->process($moniker => $file);
-      $file = delete $self->{assets}{$moniker};
-    }
-    $self->{assets}{$moniker} = \@files;
+    my @processed_files = $self->_process_many($moniker, @files);
+    $self->{assets}{$moniker} = \@processed_files;
   }
 
   $self;
 }
 
+sub _process_many {
+  my($self, $moniker, @files) = @_;
+  my %extensions = (
+    less => 'css',
+    sass => 'css',
+    scss => 'css',
+    coffee => 'js',
+  );
+  for my $file (@files) {
+    my ($extension) = $file =~ /\.(\w+)$/;
+    next unless exists $extensions{$extension};
+    my $target_ext = $extensions{$extension};
+
+    my $moniker = basename $file;
+    $moniker =~ s/\.\w+$/.$target_ext/;
+    $self->process($moniker => $file);
+    $file = delete $self->{assets}{$moniker};
+  }
+  return @files;
+}
+
 =head2 expand
 
   $bytestream = $self->expand($c, $moniker);
 
 This method will return one tag for each asset defined by the "$moniker".
 
-Will also run L</less> or L</sass> on the files to convert them to css, which
-the browser understand.
+Will also run L</less>, L</sass> or L</coffee> on the files to convert them to
+css or js, which the browser understands. (With L</MOJO_ASSETPACK_NO_CACHE>
+enabled, this is done each time on expand; with it disabled, this is done once
+when the asset is added.)
 
 The returning bytestream will contain style or script tags.
 
@@ -192,11 +227,20 @@ sub expand {
   if(!ref $files) {
     return b "<!-- Cannot expand $moniker -->";
   }
-  elsif($moniker =~ /\.js/) {
-    return b join "\n", map { $c->javascript($_) } @$files;
+
+  my @processed_files;
+  if ($self->_no_cache) {
+    @processed_files = $self->_process_many($moniker, @$files);
   }
   else {
-    return b join "\n", map { $c->stylesheet($_) } @$files;
+    @processed_files = @$files;
+  }
+
+  if($moniker =~ /\.js/) {
+    return b join "\n", map { $c->javascript($_) } @processed_files;
+  }
+  else {
+    return b join "\n", map { $c->stylesheet($_) } @processed_files;
   }
 }
 
@@ -238,7 +282,7 @@ sub process {
   $md5_sum = Mojo::Util::md5_sum(join '', map { $content->{$_} = $self->_slurp } @files);
 
   $out_file = $moniker;
-  $out_file =~ s/\.(\w{1,4})$/-$md5_sum.$1/;
+  $out_file =~ s/\.(\w+)$/-$md5_sum.$1/;
 
   if($self->{static}->file(catfile 'packed', $out_file)) {
     $self->{log}->debug("Using existing asset for $moniker");
@@ -247,7 +291,7 @@ sub process {
   }
 
   for my $file (@files) {
-    next if $file =~ /\.(\w{1,4})$/ and $self->preprocessors->has_subscribers($1);
+    next if $file =~ /\.(\w+)$/ and $self->preprocessors->has_subscribers($1);
     push @missing, $file; # will also contain files without extensions
   }
 
@@ -259,7 +303,7 @@ sub process {
   Mojo::Util::spurt(
     join('',
       map {
-        /\.(\w{1,4})$/; # checked in @missing loop
+        /\.(\w+)$/; # checked in @missing loop
         $self->preprocessors->process($1, $self, \$content->{$_}, $_);
         $content->{$_};
       } @files
@@ -292,6 +336,7 @@ sub register {
 
   $self->minify($minify);
   $self->preprocessors->detect unless $config->{no_autodetect};
+  $self->_no_cache($ENV{MOJO_ASSETPACK_NO_CACHE});
 
   $self->{assets} = {};
   $self->{log} = $app->log;
